@@ -1,6 +1,7 @@
 __version__ = '0.0.2'
 
 from collections import OrderedDict, MutableMapping, MutableSequence, Sequence
+from copy import deepcopy
 
 
 sentinel = object()
@@ -13,8 +14,9 @@ class Error(Exception):
 class Field:
     name = None
 
-    def __init__(self, *, default=sentinel):
+    def __init__(self, *, default=sentinel, const=False):
         self.default = default
+        self.const = const
 
     def set_name(self, name):
         assert self.name is None, self.name
@@ -40,9 +42,12 @@ class Field:
             raise AttributeError("'{.name}' is not initialized".format(self))
         return ret, ret
 
-    def __set__(self, instance, value):
+    def __set__(self, instance, value, check_const=True):
         name = self.name
         assert name
+        if check_const and self.const:
+            raise AttributeError(
+                "Constant '{.name}' cannot be set".format(self))
         ret, plainval = self.setter(value)
         instance.__dict__[name] = ret
         instance._plain_[name] = plainval
@@ -52,12 +57,12 @@ class Field:
 
 
 class FieldComp(Field):
-    def __init__(self, type, *, default=sentinel):
+    def __init__(self, type, *, default=sentinel, const=False):
         if default is not None and default is not sentinel:
             if not isinstance(default, type):
                 raise TypeError("an {} is required, got {}".format(
                         self.type.__name__, type(default).__name__))
-        super().__init__(default=default)
+        super().__init__(default=default, const=const)
         self.type = type
 
     def setter(self, value):
@@ -256,7 +261,7 @@ class Component(metaclass=ComponentMeta):
         fields = self._fields_
         for k, v in kwargs.items():
             if k in names:
-                fields[k].__set__(self, v)
+                fields[k].__set__(self, v, check_const=False)
         missing = names - set(kwargs.keys())
         if not missing:
             return
@@ -280,3 +285,13 @@ class Component(metaclass=ComponentMeta):
 
     def as_plain(self):
         return self._plain_
+
+    def clone(self, **kwargs):
+        ret = self.__class__(**deepcopy(self.as_plain()))
+        fields = self._fields_
+        for k, v in kwargs.items():
+            field = fields.get(k)
+            if field is None:
+                raise Error("Unknown field '{}' cannot be set".format(k))
+            field.__set__(ret, v, check_const=False)
+        return ret
